unit PBarTransparent;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Consts;

type
  TpbtOrientation = (pbtHorizontal, pbtVertical);
  TpbtStyle = (pbtDash, pbtSolid);

  TPBarTransparent = class(TGraphicControl)
  private
    FPicture: TPicture;
    FStretch: Boolean;
    FTransparent: Boolean;
    FDrawing: Boolean;
    FMin: Longint;
    FMax: Longint;
    FPosition: Longint;
    FOrientation: TpbtOrientation;
    FStyle: TpbtStyle;
    FBackColor: TColor;
    FColor: TColor;
    procedure PictureChanged(Sender: TObject);
    procedure SetOrientation(Value: TpbtOrientation);
    procedure SetStyle(Value: TpbtStyle);
    procedure SetColor(Value: TColor);
    procedure SetMin(Value: Longint);
    procedure SetMax(Value: Longint);
    procedure SetPosition(Value: Longint);
  protected
    function DestRect: TRect;
    function DoPaletteChange: Boolean;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property Color: TColor read FColor write SetColor default clBlue;
    property Orientation: TpbtOrientation read FOrientation write SetOrientation default pbtHorizontal;
    property Style: TpbtStyle read FStyle write SetStyle default pbtDash;
    property Min: Longint read FMin write SetMin default 0;
    property Max: Longint read FMax write SetMax default 100;
    property Position: Longint read FPosition write SetPosition;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TPBarTransparent]);
end;

constructor TPBarTransparent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FMin := 0;
  FMax := 100;
  FPosition := 0;
  FOrientation := pbtHorizontal;
  FStyle := pbtDash;
  FBackColor := clWhite;
  FColor := clBlue;
  Width := 150;
  Height := 13;
  FTransparent := True;
  FStretch := True;
  FPicture.Bitmap.TransparentColor := FBackColor;
end;

destructor TPBarTransparent.Destroy;
begin
  FPicture.Free;
  inherited Destroy;
end;

function TPBarTransparent.DestRect: TRect;
begin
  if FStretch then
    Result := ClientRect
  else
    Result := Rect(0, 0, FPicture.Width, FPicture.Height);
end;

procedure TPBarTransparent.Paint;
var
  Save: Boolean;
  Xstart, Ystart: Integer;
  Xdest, Ydest: Integer;
  Step: Integer;
  RelPos: Integer;
  j: Integer;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;
  Save := FDrawing;
  FDrawing := True;
  try
    Step := FMax - FMin;
    RelPos := FPosition - FMin;
//pulisci
    if (FOrientation = pbtHorizontal) then
    begin
      FPicture.Bitmap.Width := Width;
      FPicture.Bitmap.Height := 1;
    end
    else
    begin
      FPicture.Bitmap.Width := 1;
      FPicture.Bitmap.Height := Height;
    end;
    FPicture.Bitmap.TransparentColor := FBackColor;
    FPicture.Bitmap.Canvas.Brush.Color := FBackColor;
    FPicture.Bitmap.Canvas.FillRect(Rect(0, 0, FPicture.Bitmap.Width, FPicture.Bitmap.Height));
//disegna
    if (FOrientation = pbtHorizontal) then
    begin
      Xstart := 0;
      Ystart := 0;
      Xdest := Xstart + Trunc(RelPos * Width / Step);
      Ydest := Ystart;
      if (FStyle = pbtDash) then
        for j := Xstart to Xdest do
        begin
          if j mod 10 < 8 then
            FPicture.Bitmap.Canvas.Pixels[j, Ystart] := FColor;
        end;
    end
    else
    begin
      Xstart := 0;
      Ystart := Height;
      Xdest := Xstart;
      Ydest := Ystart - Trunc(RelPos * Height / Step) - 1;
      if (FStyle = pbtDash) then
        for j := Ystart downto Ydest do
        begin
          if Abs(j - Ystart) mod 10 < 8 then
            FPicture.Bitmap.Canvas.Pixels[Xstart, j] := FColor;
        end;
    end;
    if (FStyle = pbtSolid) then
    begin
      FPicture.Bitmap.Canvas.MoveTo(Xstart, Ystart);
      FPicture.Bitmap.Canvas.Pen.Style := psSolid;
      FPicture.Bitmap.Canvas.Pen.Color := FColor;
      FPicture.Bitmap.Canvas.Brush.Style := bsClear;
      FPicture.Bitmap.Canvas.LineTo(Xdest, Ydest);
    end;
    with inherited Canvas do
      StretchDraw(DestRect, FPicture.Graphic);
  finally
    FDrawing := Save;
  end;
end;

function TPBarTransparent.DoPaletteChange: Boolean;
var
  ParentForm: TCustomForm;
  Tmp: TGraphic;
begin
  Result := False;
  Tmp := FPicture.Graphic;
  if Visible and (not (csLoading in ComponentState)) and (Tmp <> nil) and
    (Tmp.PaletteModified) then
  begin
    if (Tmp.Palette = 0) then
      Tmp.PaletteModified := False
    else
    begin
      ParentForm := GetParentForm(Self);
      if Assigned(ParentForm) and ParentForm.Active and Parentform.HandleAllocated then
      begin
        if FDrawing then
          ParentForm.Perform(wm_QueryNewPalette, 0, 0)
        else
          PostMessage(ParentForm.Handle, wm_QueryNewPalette, 0, 0);
        Result := True;
        Tmp.PaletteModified := False;
      end;
    end;
  end;
end;

procedure TPBarTransparent.PictureChanged(Sender: TObject);
var
  G: TGraphic;
begin
  G := FPicture.Graphic;
  if G <> nil then
  begin
    G.Transparent := FTransparent;
    if (not G.Transparent) and (FStretch or (G.Width >= Width)
      and (G.Height >= Height)) then
      ControlStyle := ControlStyle + [csOpaque]
    else
      ControlStyle := ControlStyle - [csOpaque];
    if DoPaletteChange and FDrawing then Update;
  end
  else ControlStyle := ControlStyle - [csOpaque];
  if not FDrawing then Invalidate;
end;

procedure TPBarTransparent.SetOrientation(Value: TpbtOrientation);
begin
  if Value <> FOrientation then
  begin
    FOrientation := Value;
    PictureChanged(Self);
  end;
end;

procedure TPBarTransparent.SetStyle(Value: TpbtStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    PictureChanged(Self);
  end;
end;

procedure TPBarTransparent.SetColor(Value: TColor);
begin
  if Value <> FColor then
  begin
    FColor := Value;
    PictureChanged(Self);
  end;
end;

procedure TPBarTransparent.SetMin(Value: Longint);
begin
  if Value <> FMin then
  begin
    if Value > FMax then
      if not (csLoading in ComponentState) then
        raise EInvalidOperation.CreateFmt(SOutOfRange, [-MaxInt, FMax - 1]);
    FMin := Value;
    if FPosition < Value then FPosition := Value;
    PictureChanged(Self);
  end;
end;

procedure TPBarTransparent.SetMax(Value: Longint);
begin
  if Value <> FMax then
  begin
    if Value < FMin then
      if not (csLoading in ComponentState) then
        raise EInvalidOperation.CreateFmt(SOutOfRange, [FMin + 1, MaxInt]);
    FMax := Value;
    if FPosition > Value then FPosition := Value;
    PictureChanged(Self);
  end;
end;

procedure TPBarTransparent.SetPosition(Value: Longint);
begin
  if Value < FMin then
    Value := FMin
  else if Value > FMax then
    Value := FMax;
  if FPosition <> Value then
  begin
    FPosition := Value;
    PictureChanged(Self);
  end;
end;

end.
